# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _fir1
else:
    import _fir1

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class Fir1(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fir1.Fir1_swiginit(self, _fir1.new_Fir1(*args))
    __swig_destroy__ = _fir1.delete_Fir1

    def filter(self, input: "double") -> "double":
        return _fir1.Fir1_filter(self, input)

    def lms_update(self, error: "double") -> "void":
        return _fir1.Fir1_lms_update(self, error)

    def setLearningRate(self, _mu: "double") -> "void":
        return _fir1.Fir1_setLearningRate(self, _mu)

    def getLearningRate(self) -> "double":
        return _fir1.Fir1_getLearningRate(self)

    def reset(self) -> "void":
        return _fir1.Fir1_reset(self)

    def zeroCoeff(self) -> "void":
        return _fir1.Fir1_zeroCoeff(self)

    def getCoeff(self, *args):
            if len(args) == 0 :
    # No arguments given (other than self).
    # Set the number of taps to return from the number of weights.
                    return _fir1.Fir1_getCoeff(self, self.getTaps())
            else :
    # If any other arguments are supplied, pass them through to the C++ library.
                    return _fir1.Fir1_getCoeff(self, *args)



    def setCoeff(self, coeff_data: "double const *", number_of_taps: "unsigned int const") -> "void":
        return _fir1.Fir1_setCoeff(self, coeff_data, number_of_taps)

    def getTaps(self) -> "unsigned int":
        return _fir1.Fir1_getTaps(self)

    def getTapInputPower(self) -> "double":
        return _fir1.Fir1_getTapInputPower(self)

# Register Fir1 in _fir1:
_fir1.Fir1_swigregister(Fir1)



